/* 
* Réalisation d'un réchauffeur de bougie embarqué : 3 modes de fonctionnement sont commutables à partir du Tx.
* Les modes sont sélectionnés à partir de la voie 6 du Rx.
* La position des gaz (voie 3 du Rx) commande le réchauffage à bas régime avec une tension de 0.8V.
* Il est également prévu un réchauffage ponctuel avec une tension de 0.8V lors de la remise des gaz.
* Une position démarrage permet d'alimenter la bougie avec une tension de 1.1V.
* La voie S-PORT est utilisée pour déterminer "accelMin" et "accelMax". Elle permet également le debug du programme. A noter que pour
* fonctionner correctement aucune instruction bloquante (delay, while, ...) ne doit être introduite dans le code.
* Pour la calibration noter les valeurs d'accel à partir de la télémétrie pour les gaz en position mini et maxi puis reporter ces valeurs 
* dans "accelMin" et "accelMax". "accelMin" peut être supérieur à "accelMax" le code gère.
* Si vous ne souhaitez pas effectuer la calibration, vous pouvez utiliser les valeurs par défaut 1000 et 2000 pour "accelMin" et "accelMax". Si le réchauffage
* s'effectue dans le mauvais sens, il faudra permuter les valeurs. Vous pourrez ajuster la variable "ratio" pour définir la commutation en fonction de la 
* position de la manette des gaz.
* Vous pouvez également vérifier la valeur de mode pour les 3 positions de l'interrupteur. 
* Pour des raisons de sécurité, il est préférable de séparer les alimentations de la réception et du réchauffage.
* Renseigner la tension de la batterie de réchauffage "tensionAlim" avant de raccorder la bougie. Un mauvais paramètrage entraînerait sa 
* destruction. L'alimentation de la bougie s'effectue avec une tension modulée en largeur d'impulsion à 980 Hz.
* Choisir un transistor MOSFET canal N avec une faible tension de seuil tel que l'IRLZ44 ou tout autre transistor avec un L dans la
* désignation : 
* IRL540, IRLB3034,..., pour utiliser de faibles tensions de réchauffage (1.2V ou 2.4V) avec 1 ou 2 élément NiMH type AA (pour des questions de poids).
* L'arduino sera alimenté par la batterie de réception.
* Certaines variables sont déclarées en globale pour faciliter le debug.
*
*  auteur : DL
*  version : Glopargaz
*  date : mars 2025
*
*  Branchements :
*	
*  Arduino pin    |   Rx
*  2              |   canal 6 du mode
*  3              |   canal 3 des gaz
*  4              |   S_PORT (blanc)
*  6              |   PWM vers gate du MOSFET (pré-cablage)
*  RAW            |   Vcc (rouge)
*  GND            |   GND (noir)
*
* Une résistance de pull down de 470 Ohms sera placée entre la Gate et la Source du MOSFET (entre les pins 6 et GND de l'arduino). 
*
* Les librairies seront téléchargées aux adresses suivantes :
* https://www.rcgroups.com/forums/showthread.php?2245978-FrSky-S-Port-telemetry-library-easy-to-use-and-configurable
* https://github.com/xkam1x/Arduino-PWM-Reader
*
*/
// déclaration des libairies
#include "PWM.hpp"
#include "FrSkySportSensor.h"
#include "FrSkySportSensorFcs.h"
#include "FrSkySportSingleWireSerial.h"
#include "FrSkySportTelemetry.h"
FrSkySportSensorFcs fcs;          // Si utilisation sonde FCS-40A avec ID par défault
FrSkySportTelemetry telemetry;

// déclaration constantes
PWM ch6(2);             // entrée mode sur la pin 2
PWM ch3(3);             // entrée accel sur la pin 3
const char pwmPin = 6;  // sortie PWM sur la pin 6, f = 980 Hz

// constantes électriques
const float tensionAlim = 4.2; // tension d'alimentation ************************************************************************************
const float tensionBougie = 1.1;
const float tensionBoost = 0.8;
const float tensionReprise = 0.8;

// constante PWM
const int dutyCyclemax = floor((tensionBougie/tensionAlim)*255.); // rapport cyclique maximum de la PWM

// constantes pour la calibration des gaz
const float accelMin = 1000;  // valeur d'accel pour gaz en position mini *******************************************************************
const float accelMax = 2000;  // valeur d'accel pour gaz en position maxi *******************************************************************


// déclaration des variables
float mode = 0;            // position interrupteur mode
float accel = 0;           // position manche des gaz
float accelTime1 = 0;      // première position des gaz pour reprise
float accelTime2 = 0;      // deuxième position des gaz pour reprise
float accelSeuil = 0;      // calcul du seuil
float accelCalcul = 0;     // gére accelMin > accelMax
int dutyCycle = 0;         // rapport cyclique pour générer les tensions bougie
float tensionDisplay = 0.; // tension bougie
int saisieChoix = 0;       // variable d'état

// variable pour reprise
float ecartAccel = 0;

void initialize()
{
	Serial.begin(115200); // initialise connexion série à 115200 bauds 
	ch6.begin(true);      // PWM en pin 2 pour lire le mode haut	
	ch3.begin(true);      // PWM en pin 4 pour lire le mode haut
}

void initialize_sport()
{
	// Configure la télémétrie
	telemetry.begin(FrSkySportSingleWireSerial::SOFT_SERIAL_PIN_4, &fcs); // sortie SPORT sur la pin 4
}

void sport_output()
{
	fcs.setData(mode, accel); // mode et accel
	telemetry.send();         // envoie les données de télémétrie  
}

float accelCal() // gére accelMax < accelMin
{ 
	accel = ch3.getValue(); // acquisition de la voie 3 du Rx
	if ((accelMax - accelMin)< 0)
	{
		accelCalcul = accelMax + accelMin - accel;
	}
	else
	{
		accelCalcul = accel;
	}
	return(accelCalcul);	
}

void pwm_output() 
{         
	// constantes pour le calcul du seuil
	const float ratio = 0.25;  // position des gaz pour la commutation du réchauffage *******************************************************
	const float accelSeuilmoins = accelMax + ratio * (accelMin - accelMax); // (accelMax - accelMin) < 0
	const float accelSeuilplus = accelMin + ratio * (accelMax - accelMin);  // (accelMax - accelMin) > 0
	
	// constantes pour la sélection des modes
	const float modeMin = 1200.;    // mode sans réchauffage
	const float modeMoyinf = 1400.; // mode pour réchauffage au démarrage
	const float modeMoysup = 1600.; // mode pour réchauffage au démarrage
	const float modeMax = 1800.;    // mode pour réchauffage boost
	
	// variables pour la saisie d'accelTime1 et accelTime 2
	long presentAccel1 = 0;
	long presentAccel2 = 0;
	long presentDecal = 0;
	const float timeSample = 500;  // durée d'échantillonnage des gaz ***********************************************************************
	static long previousAccel1 = 0;
	static long previousAccel2 = 0;
	static long previousDecal = 0;
	
	// variables pour la reprise
	const float accelVar = 150;     // amplitude de variation des gaz ***********************************************************************
	const long timeReprise = 2000;  // durée de réchauffage pendant la reprise **************************************************************
	long presentReprise = 0;
	static long previousReprise = 0;
	
	// saisie du mode
	mode = ch6.getValue();  // acquisition de la voie 6 du Rx
	
	// calcul d'accelSeuil
	if ((accelMax - accelMin)< 0)
	{
		accelSeuil = accelSeuilmoins;
	}
	else
	{
		accelSeuil = accelSeuilplus;
	}
	
	// Saisie d'accelTime1 et accelTime2
	if (saisieChoix == 0)
	{
		presentAccel1 = millis();				 
		if (presentAccel1 - previousAccel1 > timeSample)
		{
			accelTime1 = accelCal();  // saisie d'accelTime1
			ecartAccel = accelTime1 - accelTime2;
			previousAccel1 = presentAccel1;
			saisieChoix = 1;	
		}	
	}
	if (saisieChoix == 1)
	{
		presentDecal = millis();				 
		if (presentDecal - previousDecal > timeSample/2)
		{
			accelTime2 = accelCal();  // saisie d'accelTime2
			previousDecal = presentDecal;
		}
	}
	
	// calcul de l'écart au démarrage du programme
	
	if (millis() < 5000)
	{
		ecartAccel = 0;   // pendant 5 secondes
	}
	
	// calcul de l'ecart pour la reprise
	
	if (ecartAccel > accelVar and mode >= modeMax) // reprise qu'en mode réchauffage
	{
		saisieChoix = 3;
	}
	else
	{
		saisieChoix = 2;
	}
	
	if (saisieChoix == 2)
	{
		if (mode <= modeMin) // pas de réchauffage
		{
			dutyCycle = 0;
			tensionDisplay = 0.;
		}
		if (mode >= modeMoyinf and mode <= modeMoysup) // démarrage
		{
			if (accelTime1 <= accelSeuil)
			{
				dutyCycle = floor((tensionBougie/tensionAlim)*255.);
				tensionDisplay = tensionBougie;
			}
			else
			{
				dutyCycle = 0;
				tensionDisplay = 0.;
			}
		}
		if (mode >= modeMax)
		{
			if (accelTime2 <= accelSeuil) // réchauffage mais pas de reprise
			{
				dutyCycle = floor((tensionBoost/tensionAlim)*255.); // on réchauffe à tensionBoost
				tensionDisplay = tensionBoost;
			}
			if (accelTime2 > accelSeuil) // arrêt du réchauffage et pas de reprise
			{
				dutyCycle = 0;
				tensionDisplay = 0.;
			}
		}
		saisieChoix = 0;
	}
	
	if (saisieChoix == 3) // réchauffage et reprise
	{
		presentReprise = millis();
		if (presentReprise - previousReprise > timeReprise)
		{
			saisieChoix = 0;
			previousReprise = presentReprise;
		}
		else
		{
			dutyCycle = floor((tensionReprise/tensionAlim)*255.); // on réchauffe à tensionReprise	
			tensionDisplay = tensionReprise;
		}
	}
	
	if (dutyCycle <= dutyCyclemax) // protection de la bougie contre les surtensions
	{
		analogWrite(pwmPin, dutyCycle);
	}
}

void setup() 
{ 
	// déclaration des constantes
	pinMode(6, OUTPUT); // sortie vers la gate du MOSFET
	initialize();
	initialize_sport();
}

void loop() 
{
	pwm_output();
	sport_output();
}
